package com.atguigu.tingshu.common.cache;

import cn.hutool.core.util.RandomUtil;
import com.atguigu.tingshu.common.constant.RedisConstant;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.stream.Collectors;

/**
 * @author: atguigu
 * @create: 2025-03-19 11:33
 */
@Slf4j
@Component
@Aspect
public class GuiGuCacheAspect {

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private RedissonClient redissonClient;


    /**
     * 自定义缓存注解增强
     *
     * @param joinPoint
     * @param guiGuCache
     * @return
     * @throws Throwable
     */
    @Around("@annotation(guiGuCache)")
    public Object around(ProceedingJoinPoint joinPoint, GuiGuCache guiGuCache) throws Throwable {
        try {
            //1.优先从Redis缓存中获取业务数据,命中缓存直接返回
            //1.1 构建业务key 形式：前缀:方法参数值(多个参数用_拼接)
            String params = "none";
            if (joinPoint.getArgs() != null && joinPoint.getArgs().length > 0) {
                params = Arrays.asList(joinPoint.getArgs())
                        .stream()
                        .map(Object::toString)
                        .collect(Collectors.joining("_"));
            }
            String dataKey = guiGuCache.prefix() + params;
            //1.2 从Redis中获取业务数据
            Object result = redisTemplate.opsForValue().get(dataKey);
            if (result != null) {
                return result;
            }
            //2.获取分布式锁
            //2.1 基于RedissonClient对象获取锁对象 入参：业务key:lock 锁粒度小一些
            String lockKey = dataKey + RedisConstant.CACHE_LOCK_SUFFIX;
            RLock lock = redissonClient.getLock(lockKey);

            //2.2 尝试获取分布式锁
            boolean flag = lock.tryLock();
            if (flag) {
                //3 获取锁成功，执行目标方法，将查库结果放入缓存，释放锁
                try {
                    //3.1 执行被自定义缓存注解修饰方法（查询数据方法）
                    result = joinPoint.proceed();
                    //3.2 将查库结果放入缓存中
                    Long ttl = guiGuCache.ttl() + RandomUtil.randomInt(50, 1000);
                    redisTemplate.opsForValue().set(dataKey, result, ttl, guiGuCache.timeUnit());
                    return result;
                } finally {
                    //3.3 释放分布式锁
                    lock.unlock();
                }
            } else {
                //4 获取锁失败，则自旋
                return this.around(joinPoint, guiGuCache);
            }
        } catch (Throwable e) {
            //5.兜底逻辑，从数据库中查数据返回
            log.error("自定义缓存分布式业务异常：{}", e);
            return joinPoint.proceed();
        }
    }
}
