package com.atguigu.tingshu.search.api;

import com.atguigu.tingshu.common.result.Result;
import com.atguigu.tingshu.query.search.AlbumIndexQuery;
import com.atguigu.tingshu.search.service.SearchService;
import com.atguigu.tingshu.vo.search.AlbumInfoIndexVo;
import com.atguigu.tingshu.vo.search.AlbumSearchResponseVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Tag(name = "搜索专辑管理")
@RestController
@RequestMapping("api/search")
@SuppressWarnings({"all"})
public class SearchApiController {

    @Autowired
    private SearchService searchService;


    /**
     * 手动将指定专辑保存到索引库
     * @param albumId
     * @return
     */
    @Operation(summary = "手动将指定专辑保存到索引库")
    @GetMapping("/albumInfo/upperAlbum/{albumId}")
    public Result upperAlbum(@PathVariable("albumId") Long albumId) {
        searchService.upperAlbum(albumId);
        return Result.ok();
    }


    @Operation(summary = "下架索引库专辑")
    @GetMapping("/albumInfo/lowerAlbum/{albumId}")
    public Result lowerAlbum(@PathVariable("albumId") Long albumId) {
        searchService.lowerAlbum(albumId);
        return Result.ok();
    }



    /**
     * 站内条件检索专辑接口
     *
     * @param albumIndexQuery
     * @return
     */
    @Operation(summary = "站内条件检索专辑接口")
    @PostMapping("/albumInfo")
    public Result<AlbumSearchResponseVo> search(@RequestBody AlbumIndexQuery albumIndexQuery) {
        AlbumSearchResponseVo vo = searchService.search(albumIndexQuery);
        return Result.ok(vo);
    }

    /**
     * 得到置顶三级分类热门专辑TOP6列表
     * @param category1Id
     * @return [{baseCategory3:{催眠音乐分类信息},list:[{专辑1},{专辑2}]},{},{}]
     */
    @Operation(summary = "得到置顶三级分类热门专辑TOP6列表")
    @GetMapping("/albumInfo/channel/{category1Id}")
    public Result<List<Map<String, Object>>> channel(@PathVariable("category1Id") Long category1Id) {
        List<Map<String, Object>> list = searchService.getBaseCategoryListByCategory1Id(category1Id);
        return Result.ok(list);
    }

    /**
     * 根据用户已录入字符返回提示词列表,自动补全效果
     * @param keyword
     * @return
     */
    @Operation(summary = "根据用户已录入字符返回提示词列表")
    @GetMapping("/albumInfo/completeSuggest/{keyword}")
    public Result<List<String>> completeSuggest(@PathVariable("keyword") String keyword) {
        List<String> list = searchService.completeSuggest(keyword);
        return Result.ok(list);
    }


    /**
     * 手动更新Redis中小时榜数据
     * @return
     */
    @Operation(summary = "手动更新Redis中小时榜数据")
    @GetMapping("/albumInfo/updateLatelyAlbumRanking")
    public Result updateLatelyAlbumRanking(){
        searchService.updateLatelyAlbumRanking();
        return Result.ok();
    }


    /**
     * 获取不同分类下不同排行维度TOP20
     * @param category1Id
     * @param dimension
     * @return
     */
    @Operation(summary = "获取不同分类下不同排行维度TOP20")
    @GetMapping("/albumInfo/findRankingList/{category1Id}/{dimension}")
    public Result<List<AlbumInfoIndexVo>> getRankingList(@PathVariable Long category1Id, @PathVariable String dimension) {
        List<AlbumInfoIndexVo> list = searchService.getRankingList(category1Id, dimension);
        return Result.ok(list);
    }
}

