package com.atguigu.tingshu.user.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.atguigu.tingshu.common.constant.SystemConstant;
import com.atguigu.tingshu.common.rabbit.constant.MqConst;
import com.atguigu.tingshu.common.rabbit.service.RabbitService;
import com.atguigu.tingshu.common.util.AuthContextHolder;
import com.atguigu.tingshu.common.util.MongoUtil;
import com.atguigu.tingshu.model.user.UserListenProcess;
import com.atguigu.tingshu.user.service.UserListenProcessService;
import com.atguigu.tingshu.vo.album.TrackStatMqVo;
import com.atguigu.tingshu.vo.user.UserListenProcessVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import static com.atguigu.tingshu.common.constant.RedisConstant.USER_TRACK_REPEAT_STAT_PREFIX;
import static com.atguigu.tingshu.common.util.MongoUtil.MongoCollectionEnum.USER_LISTEN_PROCESS;

@Service
@SuppressWarnings({"all"})
public class UserListenProcessServiceImpl implements UserListenProcessService {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private RabbitService rabbitService;

    /**
     * 从MongoDB获取声音播放进度
     *
     * @param userId  用户ID
     * @param trackId 声音ID
     * @return
     */
    @Override
    public BigDecimal getTrackBreakSecond(Long userId, Long trackId) {
        //1.获取当前用户的播放进度集合名称 userListenProcess:用户ID
        String collectionName = MongoUtil.getCollectionName(USER_LISTEN_PROCESS, userId);

        //2.根据用户ID+声音ID查询播放进度
        Query query = new Query(Criteria.where("userId").is(userId).and("trackId").is(trackId));
        UserListenProcess userListenProcess = mongoTemplate.findOne(query, UserListenProcess.class, collectionName);

        //3.如果存在记录则返回上次播放时间
        if (userListenProcess != null) {
            return userListenProcess.getBreakSecond();
        }
        //4.如果不存在记录则返回0
        return BigDecimal.valueOf(0);
    }

    /**
     * 业务需求：更新播放进度
     * 当前用户收听某个声音播放进度前端使用定时器每隔10S进行调用
     *
     * @param userListenProcessVo
     * @return
     */
    @Override
    public void updateListenProcess(UserListenProcessVo userListenProcessVo) {
        Long userId = AuthContextHolder.getUserId();
        //1.获取当前用户的播放进度集合名称 userListenProcess:用户ID
        String collectionName = MongoUtil.getCollectionName(USER_LISTEN_PROCESS, userId);

        //2.根据声音ID跟用户ID查询播放进度
        Query query = new Query(Criteria.where("userId").is(userId).and("trackId").is(userListenProcessVo.getTrackId()));
        UserListenProcess userListenProcess = mongoTemplate.findOne(query, UserListenProcess.class, collectionName);

        //3.如果播放记录为空 则保存一条播放进度
        userListenProcessVo.setBreakSecond(userListenProcessVo.getBreakSecond().setScale(0, RoundingMode.HALF_UP));
        if (userListenProcess == null) {
            //3.1 将VO中参数拷贝到PO对象
            userListenProcess = BeanUtil.copyProperties(userListenProcessVo, UserListenProcess.class);
            //3.2 设置用户ID
            userListenProcess.setCreateTime(new Date());
            userListenProcess.setUpdateTime(new Date());
            userListenProcess.setUserId(userId);
        } else {
            //4.如果播放记录存在 则更新播放进度
            userListenProcess.setBreakSecond(userListenProcessVo.getBreakSecond());
            userListenProcess.setUpdateTime(new Date());
        }
        mongoTemplate.save(userListenProcess, collectionName);

        //5.TODO 更新声音/专辑统计数值：播放 更新目标数据源：1.MySQL(专辑服务) 2.ElasticSearch(搜索服务)
        //5.1 确保某个用户,当日内只能更新一次播放量
        //5.1.1 构建key形式：前缀:用户ID:声音ID
        String redisKey = USER_TRACK_REPEAT_STAT_PREFIX + userId + ":" + userListenProcessVo.getTrackId();
        //5.1.2 尝试采用set nx命令存入Redis 有效时间：当日结束时间-当前时间
        long ttl = DateUtil.endOfDay(new Date()).getTime() - System.currentTimeMillis();
        Boolean flag = redisTemplate.opsForValue().setIfAbsent(redisKey, "1", ttl, TimeUnit.MILLISECONDS);
        //5.2 如果是当日内首次 基于MQ消息 则更新播放量
        if (flag) {
            //5.2.1 构建更新统计数值VO对象
            TrackStatMqVo mqVo = new TrackStatMqVo();
            mqVo.setAlbumId(userListenProcessVo.getAlbumId());
            mqVo.setTrackId(userListenProcessVo.getTrackId());
            mqVo.setStatType(SystemConstant.TRACK_STAT_PLAY);
            mqVo.setCount(1);
            //生成消息唯一标识：目的在消费者端确保幂等性
            String msgId = IdUtil.randomUUID();
            mqVo.setBusinessNo(msgId);
            //5.2.2 发送MQ消息通知专辑服务、搜索服务各自更新各自库中统计数值
            rabbitService.sendMessage(MqConst.EXCHANGE_TRACK, MqConst.ROUTING_TRACK_STAT_UPDATE, mqVo);
        }
    }

    /**
     * 获取最近播放专辑及声音
     *
     * @param userId
     * @return
     */
    @Override
    public Map<String, Long> getLatelyTrack(Long userId) {
        //1.获取当前用户的播放进度集合名称 userListenProcess:用户ID
        String collectionName = MongoUtil.getCollectionName(USER_LISTEN_PROCESS, userId);

        //2.根据声音ID跟用户ID查询播放进度
        Query query = new Query(Criteria.where("userId").is(userId));
        query.with(Sort.by(Sort.Direction.DESC, "updateTime"));
        query.limit(1);
        UserListenProcess listenProcess = mongoTemplate.findOne(query, UserListenProcess.class, collectionName);
        if (listenProcess != null) {
            Map<String, Long> map = new HashMap<>();
            map.put("albumId", listenProcess.getAlbumId());
            map.put("trackId", listenProcess.getTrackId());
            return map;
        }
        return null;
    }

    public static void main(String[] args) {
        System.out.println();
    }
}
